@@ -147,6 +147,20 @@ spdy_cb_send(spdylay_session *session,
   ssize_t rv;
   connection = (struct SPDY_Connection*)user_data;
   connection->want_io = IO_NONE;
+  
+  if(glob_opt.ignore_rst_stream
+    && 16 == length
+    && 0x80 == data[0]
+    && 0x00 == data[2]
+    && 0x03 == data[3]
+    )
+  {
+    PRINT_INFO2("ignoring RST_STREAM for stream_id %i %i %i %i", data[8], data[9], data[10], data[11]);
+    glob_opt.ignore_rst_stream = false;
+    return 16;
+  }
+  glob_opt.ignore_rst_stream = false;
+  
   if(connection->is_tls)
   {
     ERR_clear_error();
@@ -269,7 +283,7 @@ spdy_cb_recv(spdylay_session *session,
 
 
 static void
-spdy_cb_on_ctrl_send(spdylay_session *session,
+spdy_cb_before_ctrl_send(spdylay_session *session,
                     spdylay_frame_type type,
                     spdylay_frame *frame,
                     void *user_data)
@@ -288,6 +302,15 @@ spdy_cb_on_ctrl_send(spdylay_session *session,
       ++proxy->spdy_connection->streams_opened;
       PRINT_INFO2("opening stream: str open %i; %s", glob_opt.streams_opened, proxy->url);
       break;
+    case SPDYLAY_RST_STREAM:
+      //try to ignore duplicate RST_STREAMs
+      //TODO this will ignore RST_STREAMs also for bogus data
+      glob_opt.ignore_rst_stream = NULL==spdylay_session_get_stream_user_data(session, frame->rst_stream.stream_id);
+      PRINT_INFO2("sending RST_STREAM for %i; ignore %i; status %i",
+        frame->rst_stream.stream_id,
+        glob_opt.ignore_rst_stream,
+        frame->rst_stream.status_code);
+    break;
     default:
       break;
   }
@@ -325,7 +348,11 @@ spdy_cb_on_ctrl_recv(spdylay_session *session,
 
   proxy = spdylay_session_get_stream_user_data(session, stream_id);
   if(NULL == proxy)
-    DIE("no proxy obj");
+  {
+    PRINT_INFO2("received frame type %i for unkonwn stream id %i", type, stream_id);
+    return;
+    //DIE("no proxy obj");
+  }
 
   switch(type) {
     case SPDYLAY_SYN_REPLY:
@@ -370,14 +397,17 @@ spdy_cb_on_stream_close(spdylay_session *session,
   
   --glob_opt.streams_opened;
   --proxy->spdy_connection->streams_opened;
-  PRINT_INFO2("closing stream: str opened %i", glob_opt.streams_opened);
-  
-  DLL_remove(proxy->spdy_connection->proxies_head, proxy->spdy_connection->proxies_tail, proxy);
-  
+  PRINT_INFO2("closing stream: str opened %i; remove proxy %i", glob_opt.streams_opened, proxy->id);
+   
+  DLL_remove(proxy->spdy_connection->proxies_head, proxy->spdy_connection->proxies_tail, proxy); 
   if(proxy->http_active)
+  {
     proxy->spdy_active = false;
+  }
   else
+  {
     free_proxy(proxy);
+  }
 }
 
 
@@ -462,7 +492,7 @@ spdy_setup_spdylay_callbacks(spdylay_session_callbacks *callbacks)
   memset(callbacks, 0, sizeof(spdylay_session_callbacks));
   callbacks->send_callback = spdy_cb_send;
   callbacks->recv_callback = spdy_cb_recv;
-  callbacks->on_ctrl_send_callback = spdy_cb_on_ctrl_send;
+  callbacks->before_ctrl_send_callback = spdy_cb_before_ctrl_send;
   callbacks->on_ctrl_recv_callback = spdy_cb_on_ctrl_recv;
   callbacks->on_stream_close_callback = spdy_cb_on_stream_close;
   callbacks->on_data_chunk_recv_callback = spdy_cb_on_data_chunk_recv;
@@ -615,6 +645,7 @@ spdy_socket_set_tcp_nodelay(int fd)
 /*
  * Update |pollfd| based on the state of |connection|.
  */
+ /*
 void
 spdy_ctl_poll(struct pollfd *pollfd,
               struct SPDY_Connection *connection)
@@ -630,7 +661,7 @@ spdy_ctl_poll(struct pollfd *pollfd,
   {
     pollfd->events |= POLLOUT;
   }
-}
+}*/
 
 
 /*
@@ -766,12 +797,28 @@ spdy_connect(const struct URI *uri,
 void
 spdy_free_connection(struct SPDY_Connection * connection)
 {
+  struct Proxy *proxy;
+  struct Proxy *proxy_next;
+  
   if(NULL != connection)
   {
+    for(proxy = connection->proxies_head; NULL != proxy; proxy=proxy_next)
+    {
+      proxy_next = proxy->next;
+      DLL_remove(connection->proxies_head, connection->proxies_tail, proxy);
+      proxy->spdy_active = false;
+      proxy->spdy_error = true;
+      PRINT_INFO2("spdy_free_connection for id %i", proxy->id);
+      if(!proxy->http_active)
+      {
+        free_proxy(proxy);
+      }
+    }
     spdylay_session_del(connection->session);
     SSL_free(connection->ssl);
     free(connection->host);
     free(connection);
+    //connection->session = NULL;
   }
 }
 
@@ -835,12 +882,15 @@ spdy_request(const char **nv,
   if(ret != 0) {
     spdy_diec("spdylay_spdy_submit_request", ret);
   }
+  PRINT_INFO2("adding proxy %i", proxy->id);
+  if(NULL != connection->proxies_head)
+    PRINT_INFO2("before proxy %i", connection->proxies_head->id);
   DLL_insert(connection->proxies_head, connection->proxies_tail, proxy);
   
   return ret;
 }
 
-
+/*
 void
 spdy_get_pollfdset(struct pollfd fds[],
                    struct SPDY_Connection *connections[],
@@ -864,6 +914,7 @@ spdy_get_pollfdset(struct pollfd fds[],
       
       for(proxy = glob_opt.spdy_connection->proxies_head; NULL != proxy; proxy=proxy->next)
       {
+        abort();
         DLL_remove(glob_opt.spdy_connection->proxies_head, glob_opt.spdy_connection->proxies_tail, proxy);
         proxy->spdy_active = false;
       }
@@ -893,6 +944,7 @@ spdy_get_pollfdset(struct pollfd fds[],
       
       for(proxy = connection->proxies_head; NULL != proxy; proxy=proxy->next)
       {
+        abort();
         DLL_remove(connection->proxies_head, connection->proxies_tail, proxy);
         proxy->spdy_active = false;
       }
@@ -910,7 +962,7 @@ spdy_get_pollfdset(struct pollfd fds[],
   //, "TODO max num of conn reached; close something"
   assert(NULL == connection);
 }
-
+*/
 
 int
 spdy_get_selectfdset(fd_set * read_fd_set,
@@ -921,7 +973,7 @@ spdy_get_selectfdset(fd_set * read_fd_set,
                       nfds_t *real_size)
 {
   struct SPDY_Connection *connection;
-  struct Proxy *proxy;
+  struct SPDY_Connection *next_connection;
   bool ret;
   int maxfd = 0;
   
@@ -936,14 +988,9 @@ spdy_get_selectfdset(fd_set * read_fd_set,
 				 except_fd_set, glob_opt.spdy_connection);
     if(!ret)
     {
-      //PRINT_INFO("TODO drop connection");
       glob_opt.streams_opened -= glob_opt.spdy_connection->streams_opened;
       
-      for(proxy = glob_opt.spdy_connection->proxies_head; NULL != proxy; proxy=proxy->next)
-      {
-        DLL_remove(glob_opt.spdy_connection->proxies_head, glob_opt.spdy_connection->proxies_tail, proxy);
-        proxy->spdy_active = false;
-      }
+      PRINT_INFO("spdy_free_connection in spdy_get_selectfdset");
       spdy_free_connection(glob_opt.spdy_connection);
       glob_opt.spdy_connection = NULL;
     }
@@ -963,18 +1010,15 @@ spdy_get_selectfdset(fd_set * read_fd_set,
     ret = spdy_ctl_select(read_fd_set,
 				 write_fd_set, 
 				 except_fd_set, connection);
+         
+    next_connection = connection->next;
     if(!ret)
     {
-      //PRINT_INFO("TODO drop connection");
       glob_opt.streams_opened -= connection->streams_opened;
       DLL_remove(glob_opt.spdy_connections_head, glob_opt.spdy_connections_tail, connection);
       glob_opt.total_spdy_connections--;
       
-      for(proxy = connection->proxies_head; NULL != proxy; proxy=proxy->next)
-      {
-        DLL_remove(connection->proxies_head, connection->proxies_tail, proxy);
-        proxy->spdy_active = false;
-      }
+      PRINT_INFO("spdy_free_connection in spdy_get_selectfdset");
       spdy_free_connection(connection);
     }
     else
@@ -983,7 +1027,7 @@ spdy_get_selectfdset(fd_set * read_fd_set,
       ++(*real_size);
       if(maxfd < connection->fd) maxfd = connection->fd;
     }
-    connection = connection->next;
+    connection = next_connection;
   }
   
   //, "TODO max num of conn reached; close something"
@@ -992,7 +1036,7 @@ spdy_get_selectfdset(fd_set * read_fd_set,
   return maxfd;
 }
 
-
+/*
 void
 spdy_run(struct pollfd fds[],
          struct SPDY_Connection *connections[],
@@ -1029,9 +1073,13 @@ spdy_run(struct pollfd fds[],
         }
         for(proxy = connections[i]->proxies_head; NULL != proxy; proxy=proxy->next)
         {
+        abort();
           DLL_remove(connections[i]->proxies_head, connections[i]->proxies_tail, proxy);
           proxy->spdy_active = false;
+          proxy->spdy_error = true;
+          PRINT_INFO2("spdy_free_connection for id %i", proxy->id);
         }
+        PRINT_INFO("spdy_free_connection in loop");
         spdy_free_connection(connections[i]);
       }
     }
@@ -1039,6 +1087,7 @@ spdy_run(struct pollfd fds[],
       PRINT_INFO("not called");
   }
 }
+*/
 
 void
 spdy_run_select(fd_set * read_fd_set,
@@ -1049,7 +1098,6 @@ spdy_run_select(fd_set * read_fd_set,
 {
   int i;
   int ret;
-  struct Proxy *proxy;
   
   for(i=0; i<size; ++i)
   {
@@ -1071,11 +1119,7 @@ spdy_run_select(fd_set * read_fd_set,
           DLL_remove(glob_opt.spdy_connections_head, glob_opt.spdy_connections_tail, connections[i]);
           glob_opt.total_spdy_connections--;
         }
-        for(proxy = connections[i]->proxies_head; NULL != proxy; proxy=proxy->next)
-        {
-          DLL_remove(connections[i]->proxies_head, connections[i]->proxies_tail, proxy);
-          proxy->spdy_active = false;
-        }
+        PRINT_INFO("in spdy_run_select");
         spdy_free_connection(connections[i]);
       }
     }
